import { Observable } from '../Observable';
import { ReplaySubject } from '../ReplaySubject';
/**
 * @method shareReplay
 * @owner Observable
 */
export function shareReplay(bufferSize, windowTime, scheduler) {
    let subject;
    let refCount = 0;
    let subscription;
    let hasError = false;
    let isComplete = true;
    return (source) => new Observable(observer => {
        refCount++;
        if (!subject || hasError) {
            hasError = false;
            subject = new ReplaySubject(bufferSize, windowTime, scheduler);
            subscription = source.subscribe({
                next(value) { subject.next(value); },
                error(err) {
                    hasError = true;
                    subject.error(err);
                },
                complete() {
                    isComplete = true;
                    subject.complete();
                },
            });
        }
        const innerSub = subject.subscribe(observer);
        return () => {
            refCount--;
            innerSub.unsubscribe();
            if (subscription && refCount === 0 && !isComplete) {
                subscription.unsubscribe();
            }
        };
    });
}
;
//# sourceMappingURL=shareReplay.js.map